using System;
using System.Text.RegularExpressions;

namespace Smdn.Applications.DirectoryAnalyzer.Analysis
{
  using FileSystem;

  public class AnalysisContext : TaskContext, ITaskContext
  {
    public AnalysisContext(Directory analysisDirectory)
    {
      this.parameter = new AnalysisParameter(analysisDirectory);
      this.status = new AnalysisStatus();
      this.result = new AnalysisResult();
    }

#region parameters
    private AnalysisParameter parameter = null;

    public AnalysisParameter Parameter
    {
      get
      {
        return this.parameter;
      }
    }
    
    public class AnalysisParameter : TaskParameter
    {
      private Directory analysisDirectory = null;

      public Directory AnalysisDirectory
      {
        get
        {
          return this.analysisDirectory;
        }
      }

      public string OutputFile = string.Empty;
      public uint AnalysisDepth = 0;
      public bool PreAnalyze = true;
      public Regex ExcludeRegex = null;

      public AnalysisParameter(Directory analysisDirectory)
      {
        this.analysisDirectory = analysisDirectory;
      }
    }
#endregion

#region status
    private AnalysisStatus status = null;

    public AnalysisStatus Status
    {
      get
      {
        return this.status;
      }
    }

    public class AnalysisStatus : TaskStatus
    {
      public Directory.EntryCount EntryCount = new Directory.EntryCount();
      public uint AnalyzedEntryCount = 0;

      public override uint Total
      {
        get
        {
          return EntryCount.TotalEntryCount;
        }
      }

      public override uint Processed
      {
        get
        {
          return this.AnalyzedEntryCount;
        }
      }
    }
#endregion

#region results
    private AnalysisResult result = null;

    public AnalysisResult Result
    {
      get
      {
        return this.result;
      }
    }

    public class AnalysisResult : TaskResult
    {
      public uint ExcludedDirectoryCount = 0;
      public Tree Tree = null;
    }
#endregion

    #region ITaskContext

    TaskContext.TaskParameter ITaskContext.Parameter
    {
      get { return this.parameter; }
    }

    TaskContext.TaskResult ITaskContext.Result
    {
      get { return this.result; }
    }

    TaskContext.TaskStatus ITaskContext.Status
    {
      get { return this.status; }
    }

    #endregion
  }
}
