using System;
using System.Text.RegularExpressions;

namespace Smdn.Applications.DirectoryAnalyzer.Analysis
{
  using FileSystem;

  public class Analyzer
  {
    private AnalysisContext context = null;

    public Analyzer(AnalysisContext context)
    {
      if (null == context) throw new ArgumentNullException("context");

      this.context = context;
    }

    public void Analyze()
    {
      if (context.Parameter.PreAnalyze)
      {
        // ディレクトリ内を先行走査
        context.Status.StatusText = "ディレクトリ内のエントリ数を取得中";

        Directory.GetEntryCount(context.Parameter.AnalysisDirectory, true, ref context.Status.EntryCount);
      }

      // 対象ディレクトリを走査
      Tree tree = new Tree(context.Parameter.AnalysisDirectory);
      uint excludedDirectoryCount = 0;

      context.Status.StatusText = "ディレクトリを解析中";

      if (null == context.Parameter.ExcludeRegex)
      {
        // 除外するディレクトリが指定されていない場合
        tree.Scan(context.Status, null, delegate
        {
          // 一切除外しない
          return false;
        });
      }
      else
      {
        // 除外するディレクトリが指定されている場合
        tree.Scan(context.Status, context.Parameter.ExcludeRegex, delegate(Regex regex, Directory directory)
        {
          excludedDirectoryCount++;

          // 正規表現にマッチしたら除外する
          return regex.IsMatch(directory.FullName);
        });
      }

      context.Result.Tree = tree;
      context.Result.ExcludedDirectoryCount = excludedDirectoryCount;

      context.Status.StatusText = "ディレクトリの解析が完了しました";
    }
  }
}