using System;

namespace Smdn.Applications.DirectoryAnalyzer.FileSystem
{
  public enum EntryCompareMode
  {
    BySize,
    ByName,
    ByFullName,
  }

  public static class Comparison
  {
    private static int ByType(Entry x, Entry y)
    {
      if (x is Directory && y is File) return 1;
      if (x is File && y is Directory) return -1;

      return 0;
    }

    public static Comparison<Entry> GetComparer(EntryCompareMode compareMode, bool compareEntryType)
    {
      Comparison<Entry> comparer = null;

      if (compareEntryType)
      {
        switch (compareMode)
        {
          case EntryCompareMode.BySize:
            comparer = BySize;
            break;

          case EntryCompareMode.ByName:
            comparer = ByName;
            break;

          case EntryCompareMode.ByFullName:
            comparer = ByFullName;
            break;
        }
      }
      else
      {
        switch (compareMode)
        {
          case EntryCompareMode.BySize:
            comparer = BySizeIgnoreType;
            break;

          case EntryCompareMode.ByName:
            comparer = ByNameIgnoreType;
            break;

          case EntryCompareMode.ByFullName:
            comparer = ByFullNameIgnoreType;
            break;
        }
      }

      return comparer;
    }

    public static int ByName(Entry x, Entry y)
    {
      int comp = ByType(x, y);

      if (0 == comp)
      {
        return ByNameIgnoreType(x, y);
      }
      else
      {
        return comp;
      }
    }

    public static int ByNameIgnoreType(Entry x, Entry y)
    {
      return String.Compare(x.Name, y.Name);
    }

    public static int ByFullName(Entry x, Entry y)
    {
      int comp = ByType(x, y);

      if (0 == comp)
      {
        return ByFullNameIgnoreType(x, y);
      }
      else
      {
        return comp;
      }
    }

    public static int ByFullNameIgnoreType(Entry x, Entry y)
    {
      return String.Compare(x.FullName, y.FullName);
    }

    public static int BySize(Entry x, Entry y)
    {
      int comp = ByType(x, y);

      if (0 == comp)
      {
        return BySizeIgnoreType(x, y);
      }
      else
      {
        return comp;
      }
    }

    public static int BySizeIgnoreType(Entry x, Entry y)
    {
      int comp = Size.Compare(x.Size, y.Size);

      if (0 == comp)
      {
        // サイズが同じ場合は、名前で比較する
        return String.Compare(x.Name, y.Name);
      }
      else
      {
        return comp;
      }
    }
  }
}
