#if LINUX

using Mono.Unix.Native;
using System;

namespace Smdn.Applications.DirectoryAnalyzer.FileSystem
{
  using FileSystem;

  partial class Directory
  {
    private enum DirentType : byte
    {
      DT_UNKNOWN = 0,
      DT_FIFO = 1,
      DT_CHR = 2,
      DT_DIR = 4,
      DT_BLK = 6,
      DT_REG = 8,
      DT_LNK = 10,
      DT_SOCK = 12,
      DT_WHT = 14
    };

    private static void GetEntryCountWithReaddir(string dir, bool recursive, ref EntryCount entryCount)
    {
      IntPtr dirp = IntPtr.Zero;

      try
      {
        dirp = Syscall.opendir(dir);

        // can't open directory
        if (IntPtr.Zero.Equals(dirp)) return;

        for (; ; )
        {
          IntPtr result;
          Dirent dirent = new Dirent();

          if (0 != Syscall.readdir_r(dirp, dirent, out result)) break;

          // end of directory entry
          if (IntPtr.Zero.Equals(result)) break;

          // removed entry
          if (0 == dirent.d_ino) continue;

          switch((DirentType)dirent.d_type)
          {
            case DirentType.DT_DIR:
              // itself or parent directory
              if ("." == dirent.d_name || ".." == dirent.d_name) continue;

              entryCount.DirectoryCount++;

              if (recursive)
              {
                GetEntryCountWithReaddir(dir + System.IO.Path.DirectorySeparatorChar + dirent.d_name, recursive, ref entryCount);
              }
              break;

            case DirentType.DT_REG:
              entryCount.FileCount++;
              break;

            default:
              // do nothing
              break;
          }
        }
      }
      finally
      {
        if (!IntPtr.Zero.Equals(dirp))
        {
          Syscall.closedir(dirp);
        }
      }
    }
  }
}
#endif
