#if WINDOWS

using System;
using System.Runtime.InteropServices;

namespace Smdn.Applications.DirectoryAnalyzer.FileSystem
{
  using FileSystem;

  partial class Directory
  {
    private const int MAX_PATH = 260;
    private const int MAX_ALTERNATE = 14;
    private static readonly IntPtr INVALID_HANDLE_VALUE = new IntPtr(-1);

    [StructLayout(LayoutKind.Sequential)]
    private struct FILETIME
    {
      public uint dwLowDateTime;
      public uint dwHighDateTime;
    };

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
    private struct WIN32_FIND_DATA
    {
      public System.IO.FileAttributes dwFileAttributes;
      public FILETIME ftCreationTime;
      public FILETIME ftLastAccessTime;
      public FILETIME ftLastWriteTime;
      public int nFileSizeHigh;
      public int nFileSizeLow;
      public int dwReserved0;
      public int dwReserved1;
      [MarshalAs(UnmanagedType.ByValTStr, SizeConst = MAX_PATH)]
      public string cFileName;
      [MarshalAs(UnmanagedType.ByValTStr, SizeConst = MAX_ALTERNATE)]
      public string cAlternate;
    }

    [DllImport("kernel32.dll", CharSet = CharSet.Unicode)]
    private static extern IntPtr FindFirstFile(string lpFileName, out WIN32_FIND_DATA lpFindFileData);

    [DllImport("kernel32.dll", CharSet = CharSet.Unicode)]
    private static extern bool FindNextFile(IntPtr hFindFile, out WIN32_FIND_DATA lpFindFileData);

    [DllImport("kernel32.dll")]
    private static extern bool FindClose(IntPtr hFindFile);

    private static void GetEntryCountWithFind(string dir, bool recursive, ref EntryCount entryCount)
    {
      IntPtr hFindFile = INVALID_HANDLE_VALUE;

      try
      {
        WIN32_FIND_DATA findData;
        hFindFile = FindFirstFile(dir + @"\*", out findData);

        if (INVALID_HANDLE_VALUE == hFindFile) return;

        do
        {
          if (System.IO.FileAttributes.Directory == (findData.dwFileAttributes & System.IO.FileAttributes.Directory))
          {
            // ディレクトリ
            if ("." == findData.cFileName || ".." == findData.cFileName)
            {
              // do nothing
            }
            else
            {
              entryCount.DirectoryCount++;

              if (recursive)
              {
                GetEntryCountWithFind(dir + System.IO.Path.DirectorySeparatorChar + findData.cFileName, recursive, ref entryCount);
              }
            }
          }
          else
          {
            // ファイル
            entryCount.FileCount++;
            entryCount.TotalSize += (ulong)findData.nFileSizeLow + ((ulong)findData.nFileSizeHigh << 32);
          }
        }
        while (FindNextFile(hFindFile, out findData));
      }
      finally
      {
        if (INVALID_HANDLE_VALUE != hFindFile)
        {
          FindClose(hFindFile);
        }
      }
    }
  }
}
#endif
