using System;

#if LINUX
using EntryInfo = Mono.Unix.UnixFileSystemInfo;
#elif WINDOWS
using EntryInfo = System.IO.FileSystemInfo;
#endif

namespace Smdn.Applications.DirectoryAnalyzer.FileSystem
{
  public abstract class Entry
  {
    private EntryInfo info = null;

    protected Entry(EntryInfo info)
    {
      if (null == info) throw new ArgumentNullException("info");

      this.info = info;
    }

    public EntryInfo Info
    {
      get
      {
        return this.info;
      }
    }

    public abstract string FullName { get;}
    public abstract string Name { get;}
    public abstract Size Size { get;}
    public abstract bool Exists { get;}
    public abstract Directory Parent { get;}

    public override string ToString()
    {
      return string.Format("{0} ({1}B, '{2}')", this.Name, this.Size, this.FullName);
    }
  }
}
