using System;

#if LINUX
using FileInfo = Mono.Unix.UnixFileInfo;
#elif WINDOWS
using FileInfo = System.IO.FileInfo;
#endif

namespace Smdn.Applications.DirectoryAnalyzer.FileSystem
{
  public class File : Entry
  {
    public File(FileInfo file)
      : base(file)
    {
    }

    public File(string filePath)
      : base(new FileInfo(filePath))
    {
    }

    public override string FullName
    {
      get { return (base.Info as FileInfo).FullName; }
    }

    public override string Name
    {
      get { return (base.Info as FileInfo).Name; }
    }

    private Size? cachedSize = null;

    public override Size Size
    {
      get
      {
        if (cachedSize == null)
        {
          if (this.Exists)
          {
            cachedSize = new Size((base.Info as FileInfo).Length);
          }
          else
          {
            // ファイルが無くなった場合は、ひとまず0を返しておく
            cachedSize = null;

            return Size.Zero;
          }
        }
        
        return (Size)cachedSize;
      }
    }

    public override bool Exists
    {
      get
      {
        return (base.Info as FileInfo).Exists;
      }
    }

    public override Directory Parent
    {
      get
      {
        return new Directory((base.Info as FileInfo).Directory);
      }
    }
  }
}
