using System;
using System.Collections.Generic;
using System.Text;

namespace Smdn.Applications.DirectoryAnalyzer.FileSystem
{
  public struct Size
  {
    public static Size Zero = new Size(0);
    private static string[] auxUnits = new string[] { "", "k", "M", "G", "T", "P", "E" };

    private ulong size;

    public Size(long size)
    {
      this.size = (ulong)size;
    }

    public Size(ulong size)
    {
      this.size = size;
    }

#region operators
    public static Size operator +(Size x, Size y)
    {
      return new Size(x.size + y.size);
    }

    public static Size operator *(Size x, Size y)
    {
      return new Size(x.size * y.size);
    }

    public static Size operator *(Size x, ulong y)
    {
      return new Size(x.size * y);
    }

    public static Size operator /(Size x, Size y)
    {
      return new Size(x.size / y.size);
    }

    public static Size operator /(Size x, ulong y)
    {
      return new Size(x.size / y);
    }

    public static double operator /(Size x, double y)
    {
      return (double)x.size / y;
    }

    public static bool operator ==(Size x, Size y)
    {
      return (x.size == y.size);
    }

    public static bool operator !=(Size x, Size y)
    {
      return (x.size != y.size);
    }

    public static bool operator <(Size x, Size y)
    {
      return (x.size < y.size);
    }

    public static bool operator >(Size x, Size y)
    {
      return (x.size > y.size);
    }

    public override bool Equals(object obj)
    {
      if (obj is Size)
      {
        return (this.size == ((Size)obj).size);
      }
      else
      {
        return false;
      }
    }
#endregion

#region type conversion

    public static implicit operator Size(ulong x)
    {
      return new Size(x);
    }

    public static implicit operator ulong(Size x)
    {
      return x.size;
    }

    public static explicit operator double(Size x)
    {
      return (double)x.size;
    }
#endregion

    public static int Compare(Size x, Size y)
    {
      if (x.size < y.size)
      {
        return 1;
      }
      else if (x.size > y.size)
      {
        return -1;
      }
      else
      {
        return 0;
      }
    }

    public override int GetHashCode()
    {
      return base.GetHashCode();
    }

    public override string ToString()
    {
      // どの補助単位が適切か算出する(1000以上の数値には補助単位を付ける)
      int aux = 0;
      ulong s = this.size;
      ulong ss = s;

      for (; 1000 <= s; s >>= 10)
      {
        aux++;
        ss = s;
      }

      if (0 == aux)
      {
        // 接頭語なしの場合はすべての桁を表示 (小数部はない)
        return string.Format("{0:F0}", s);
      }
      else
      {
        string format = string.Empty;
        double ds = (double)ss / 1024.0;

        if (1000 <= s)
        {
          // 補助単位を一つ上のものにする
          ds /= 1024.0;
          aux += 1;
          
          format = "{0:F2}{1}";
        }
        else if (100 <= s)
        {
          format = "{0:F0}{1}";
        }
        else if (10 <= s)
        {
          format = "{0:F1}{1}";
        }
        else
        {
          format = "{0:F2}{1}";
        }

        return string.Format(format, ds, auxUnits[aux]);
      }
    }

    public string ToBinaryPrefixedString()
    {
      return string.Format("{0}iB", this.ToString());
    }

    public string ToNumericString()
    {
      return string.Format("{0:N0} Bytes", this.size);
    }
  }
}
