using System;
using System.IO;
using System.Text;

namespace Smdn.Applications.DirectoryAnalyzer.Format
{
  using Analysis;
  using FileSystem;
  using Graphing;

  partial class Formatter
  {
    /// <summary>
    /// CSV用のFormatter
    /// </summary>
    private class CsvFormatter
    {
      private FormattingContext.FormattingParameter parameter;

      public CsvFormatter(FormattingContext.FormattingParameter parameter)
      {
        this.parameter = parameter;
      }

      public void Format(TextWriter writer, FormattingContext.FormattingStatus status)
      {
        status.TotalEntryCount = (uint)parameter.FormattingElements.Length;
        status.FormattedEntryCount = 0;

        writer.WriteLine("種類,名前,容量[%],容量[Bytes],フルパス");

        foreach (IGraphElement element in parameter.FormattingElements)
        {
          string type = string.Empty;
          Entry entry = element.Entry;

          if(entry is File)
          {
            type = "f";
          }
          else if ( entry is Directory)
          {
            type = "d";
          }
          else
          {
            continue;
          }

          // Type, Name, Size(%), Size(Bytes), FullName
          writer.WriteLine("{0},\"{1}\",{2:P2},{3},\"{4}\"",
            type,
            entry.Name,
            element.Range,
            (ulong)entry.Size,
            entry.FullName
            );

          status.FormattedEntryCount++;
        }

        status.FormattedEntryCount = status.TotalEntryCount;

        writer.Flush();
      }
    }
  }
}