using System;
using System.IO;
using System.Text;
using System.Xml;

namespace Smdn.Applications.DirectoryAnalyzer.Format
{
  using Analysis;
  using Graphing;
  using FileSystem;

  partial class Formatter
  {
    /// <summary>
    /// XML用のFormatter
    /// </summary>
    private class XmlFormatter
    {
      private FormattingContext.FormattingParameter parameter;

      public XmlFormatter(FormattingContext.FormattingParameter parameter)
      {
        this.parameter = parameter;
      }

      public void Format(TextWriter writer, FormattingContext.FormattingStatus status)
      {
        Tree tree = parameter.Graph.Tree;

        status.TotalEntryCount = (uint)parameter.FormattingElements.Length;
        status.FormattedEntryCount = 0;

        XmlDocument document = new XmlDocument();

        document.AppendChild(document.CreateXmlDeclaration("1.0", null, null));

        XmlElement root = document.CreateElement("directoryAnalyzerResult");

        document.AppendChild(root);

        root.Attributes.Append(document.CreateAttribute("root")).Value = tree.RootDirectory.FullName;

        XmlElement stat = document.CreateElement("statistics");

        root.AppendChild(stat);

        stat.AppendChild(document.CreateElement("name")).InnerText = tree.RootDirectory.Name;
        stat.AppendChild(document.CreateElement("entries")).InnerText = tree.GetAllEntryCount().ToString();
        stat.AppendChild(document.CreateElement("files")).InnerText = tree.GetAllFileCount().ToString();
        stat.AppendChild(document.CreateElement("directories")).InnerText = tree.GetAllSubTreeCount().ToString();

        Size totalSize = tree.RootDirectory.Size;

        stat.AppendChild(document.CreateElement("size")).InnerText = ((ulong)totalSize).ToString();
        stat.AppendChild(document.CreateElement("sizeString")).InnerText = totalSize.ToBinaryPrefixedString();

        XmlElement result = document.CreateElement("result");

        root.AppendChild(result);

        result.Attributes.Append(document.CreateAttribute("entries")).Value = parameter.FormattingElements.Length.ToString();

        foreach (IGraphElement element in parameter.FormattingElements)
        {
          XmlElement ent;
          Entry entry = element.Entry;

          if (entry is Directory)
          {
            ent = document.CreateElement("directory");
          }
          else if (entry is File)
          {
            ent = document.CreateElement("file");
          }
          else
          {
            continue;
          }

          result.AppendChild(ent).InnerText = entry.FullName;

          Size size = entry.Size;

          ent.Attributes.Append(document.CreateAttribute("name")).Value = entry.Name;
          ent.Attributes.Append(document.CreateAttribute("size")).Value = ((ulong)size).ToString();
          ent.Attributes.Append(document.CreateAttribute("sizePercent")).Value = element.Range.ToString("P2");
          ent.Attributes.Append(document.CreateAttribute("sizeString")).Value = size.ToBinaryPrefixedString();
          ent.Attributes.Append(document.CreateAttribute("directory")).Value = entry.Parent.FullName;

          status.FormattedEntryCount++;
        }

        status.FormattedEntryCount = status.TotalEntryCount;

        XmlWriterSettings settings = new XmlWriterSettings();

        settings.Encoding = Encoding.UTF8;
        settings.Indent = true;
        settings.IndentChars = "  ";

        XmlWriter xw = XmlWriter.Create(writer, settings);

        document.WriteTo(xw);

        xw.Flush();
      }
    }
  }
}