using System;
using System.IO;

namespace Smdn.Applications.DirectoryAnalyzer.Format
{
  using Graphing;

  public enum FormatType
  {
    Xml,
    Csv,
  }

  public partial class Formatter
  {
    private FormattingContext context = null;

    public Formatter(FormattingContext context)
    {
      if (null == context) throw new ArgumentNullException("context");

      this.context = context;
    }

    public void Format()
    {
      context.Status.StatusText = "整形中";

      if (null != context.Parameter.Comparer)
      {
        Array.Sort<IGraphElement>(context.Parameter.FormattingElements, context.Parameter.Comparer);
      }

      TextWriter writer = context.Parameter.Writer;

      switch (this.context.Parameter.Type)
      {
        case FormatType.Xml:
          (new XmlFormatter(context.Parameter)).Format(writer, context.Status);
          break;

        case FormatType.Csv:
          (new CsvFormatter(context.Parameter)).Format(writer, context.Status);
          break;

        default:
          throw new NotImplementedException(string.Format("{0}は整形出来ません", context.Parameter.Type));
      }

      context.Result.Formatted = writer;

      context.Status.StatusText = "整形が完了しました";
    }
  }
}
