using System;
using System.IO;

namespace Smdn.Applications.DirectoryAnalyzer.Format
{
  using Analysis;
  using Graphing;

  public class FormattingContext : TaskContext, ITaskContext
  {
    public FormattingContext()
    {
      this.parameter = new FormattingParameter();
      this.status = new FormattingStatus();
      this.result = new FormattingResult();
    }

    #region parameters
    private FormattingParameter parameter = null;

    public FormattingParameter Parameter
    {
      get
      {
        return this.parameter;
      }
    }

    public class FormattingParameter : TaskParameter
    {
      public FormatType Type = FormatType.Xml;
      public Graph Graph = null;
      public IGraphElement[] FormattingElements = null;
      public TextWriter Writer = null;
      public Comparison<IGraphElement> Comparer = null;
    }
    #endregion

    #region status
    private FormattingStatus status = null;

    public FormattingStatus Status
    {
      get
      {
        return this.status;
      }
    }

    public class FormattingStatus : TaskStatus
    {
      public uint TotalEntryCount = 0;
      public uint FormattedEntryCount = 0;

      public override uint Total
      {
        get
        {
          return this.TotalEntryCount;
        }
      }

      public override uint Processed
      {
        get
        {
          return this.FormattedEntryCount;
        }
      }
    }
    #endregion

    #region results
    private FormattingResult result = null;

    public FormattingResult Result
    {
      get
      {
        return this.result;
      }
    }

    public class FormattingResult : TaskResult
    {
      public TextWriter Formatted = null;
    }
    #endregion

    #region ITaskContext

    TaskContext.TaskParameter ITaskContext.Parameter
    {
      get { return this.parameter; }
    }

    TaskContext.TaskResult ITaskContext.Result
    {
      get { return this.result; }
    }

    TaskContext.TaskStatus ITaskContext.Status
    {
      get { return this.status; }
    }

    #endregion
  }
}
