using System;
using System.Collections.Generic;
using System.Text;

namespace Smdn.Applications.DirectoryAnalyzer.Graphing.Disc
{
  using Analysis;
  using FileSystem;

  public class Pie : IGraphElement
  {
    private Entry entry = null;
    private Disc disc = null;
    private TreePie parent = null;
    private uint? depth = null;
    private double range = 0.0;

    public Pie(Disc disc, Entry entry, TreePie parent)
    {
      this.disc = disc;
      this.entry = entry;
      this.parent = parent;
    }

    public Entry Entry
    {
      get
      {
        return this.entry;
      }
    }

    Entry IGraphElement.Entry
    {
      get
      {
        return this.entry;
      }
    }

    public Disc Disc
    {
      get
      {
        return this.disc;
      }
    }

    Graph IGraphElement.Graph
    {
      get
      {
        return this.disc;
      }
    }

    public TreePie Parent
    {
      get
      {
        return this.parent;
      }
    }

    ITreeGraphElement IGraphElement.Parent
    {
      get
      {
        return this.parent;
      }
    }

    public Pie GetParent(uint depth)
    {
      uint d = this.Depth;

      for (Pie p = this; p != null; p = p.parent)
      {
        if (d-- == depth) return p;
      }

      return null;
    }

    public uint Depth
    {
      get
      {
        if (depth == null)
        {
          depth = 0;

          for (Pie p = this.parent; p != null; p = p.parent)
          {
            depth++;
          }
        }

        return (uint)depth;
      }
    }

    public double Range
    {
      get
      {
        return this.range;
      }
      set
      {
        this.range = value;
      }
    }

    public double Argument;
    public double Volume;
  }

  public class TreePie : Pie, ITreeGraphElement
  {
    private Tree tree = null;
    private Pie[] subPies = null;

    public Tree Tree
    {
      get
      {
        return this.tree;
      }
    }

    public Pie[] SubPies
    {
      get
      {
        return this.subPies;
      }
      set
      {
        this.subPies = value;
      }
    }

    public TreePie(Disc disc, Tree tree, TreePie parent)
      : base(disc, tree.RootDirectory, parent)
    {
      this.tree = tree;
      this.subPies = new Pie[] { };
    }
  }
}
