using System;
using System.Text.RegularExpressions;

namespace Smdn.Applications.DirectoryAnalyzer.Graphing
{
  using Analysis;
  using FileSystem;

  public enum GraphType
  {
    Disc
  }

  public interface IGraphElement
  {
    Entry Entry { get;}
    Graph Graph { get;}
    ITreeGraphElement Parent { get;}
    double Range { get;}
  }

  public interface ITreeGraphElement : IGraphElement
  {
    Tree Tree { get;}
  }

  public abstract class Graph
  {
    private GraphType type;

    public GraphType Type
    {
      get
      {
        return this.type;
      }
    }

    private Tree tree = null;

    public Tree Tree
    {
      get
      {
        return this.tree;
      }
    }

    protected Graph(GraphingContext context)
    {
      this.type = context.Parameter.Type;
      this.tree = context.Parameter.Tree;
    }

    public abstract IGraphElement[] GetAllElements();

    public abstract IGraphElement[] SearchElement(Regex regex, RegexPredicate<IGraphElement> match);

    public abstract IGraphElement[] SearchAllElement(Regex regex, RegexPredicate<IGraphElement> match);
  }
}
