using System;

namespace Smdn.Applications.DirectoryAnalyzer.Graphing
{
  using Analysis;
  using FileSystem;

  public class GraphingContext : TaskContext, ITaskContext
  {
    public GraphingContext()
    {
      this.parameter = new GraphingParameter();
      this.status = new GraphingStatus();
      this.result = new GraphingResult();
    }

    #region parameters
    private GraphingParameter parameter = null;

    public GraphingParameter Parameter
    {
      get
      {
        return this.parameter;
      }
    }

    public class GraphingParameter : TaskParameter
    {
      public GraphType Type = GraphType.Disc;
      public uint MaxDepth = 0;
      public Tree Tree = null;
      public Comparison<Entry> Comparer = Comparison.BySize;
    }
    #endregion

    #region status
    private GraphingStatus status = null;

    public GraphingStatus Status
    {
      get
      {
        return this.status;
      }
    }

    public class GraphingStatus : TaskStatus
    {
      public uint TotalEntryCount = 0;
      public uint GraphedEntryCount = 0;

      public override uint Total
      {
        get
        {
          return this.TotalEntryCount; 
        }
      }

      public override uint Processed
      {
        get 
        {
          return this.GraphedEntryCount;  
        }
      }
    }
    #endregion

    #region results
    private GraphingResult result = null;

    public GraphingResult Result
    {
      get
      {
        return this.result;
      }
    }

    public class GraphingResult : TaskResult
    {
      public Graph Graph = null;
    }
    #endregion

    #region ITaskContext

    TaskContext.TaskParameter ITaskContext.Parameter
    {
      get { return this.parameter; }
    }

    TaskContext.TaskResult ITaskContext.Result
    {
      get { return this.result; }
    }

    TaskContext.TaskStatus ITaskContext.Status
    {
      get { return this.status; }
    }

    #endregion
  }
}
