using System;
using System.Drawing;

namespace Smdn.Applications.DirectoryAnalyzer.Render
{
  public struct Hsi
  {
    private byte h;
    private byte s;
    private byte i;

    /// <summary>
    /// 色相
    /// </summary>
    public byte Hue
    {
      get
      {
        return h;
      }
    }

    /// <summary>
    /// 彩度
    /// </summary>
    public byte Saturation
    {
      get
      {
        return s;
      }
    }

    /// <summary>
    /// 明度
    /// </summary>
    public byte Intensity
    {
      get
      {
        return i;
      }
    }

    public Hsi(byte h, byte s, byte i)
    {
      this.h = h;
      this.s = s;
      this.i = i;
    }

    public Color ToColor()
    {
      byte r, g, b;

      if (s == 0)
      {
        r = g = b = i;
      }
      else
      {
        int ht = h * 6;
        int d = ht % 256;
        byte t1 = (byte)((i * (255 - s)) / 255);
        byte t2 = (byte)((i * (255 - s * d / 255)) / 255);
        byte t3 = (byte)((i * (255 - s * (255 - d) / 255)) / 255);

        switch (ht / 256)
        {
          case 0:
            r = i; g = t3; b = t1; break;
          case 1:
            r = t2; g = i; b = t1; break;
          case 2:
            r = t1; g = i; b = t3; break;
          case 3:
            r = t1; g = t2; b = i; break;
          case 4:
            r = t3; g = t1; b = i; break;
          default:
            r = i; g = t1; b = t2; break;
        }
      }

      return Color.FromArgb(0xff, r, g, b);
    }
  }
}
