using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;

namespace Smdn.Applications.DirectoryAnalyzer.Render
{
  using Graphing;

  public partial class Renderer
  {
    RenderingContext context = null;

    public Renderer(RenderingContext context)
    {
      if (null == context) throw new ArgumentNullException("context");

      this.context = context;
    }

    public void Render()
    {
      context.Status.StatusText = "グラフを描画中";

      RenderingContext.RenderingParameter param = context.Parameter;

      Bitmap bitmap = new Bitmap((int)param.OutputWidth, (int)param.OutputHeight, PixelFormat.Format32bppArgb);

      context.Result.GraphImage = bitmap;

      using (Graphics g = Graphics.FromImage(bitmap))
      {
        // 透明でクリア
        g.Clear(Color.Transparent);

        g.SmoothingMode = param.SmoothingMode;
        g.InterpolationMode = param.InterpolationMode;
        g.TextRenderingHint = param.TextRenderingHint;

        // 描画領域
        switch (param.Graph.Type)
        {
          case GraphType.Disc:
            using (DiscRenderer renderer = new DiscRenderer(param))
            {
              renderer.Render(g, context.Status);
            }
            break;

          default:
            // TODO: 円グラフ以外の形式はまだ
            break;
        }
      }

      context.Status.StatusText = "グラフの描画が完了しました";
    }

    public IGraphElement GetElementAt(PointF point)
    {
      switch (context.Parameter.Graph.Type)
      {
        case GraphType.Disc:
          using (DiscRenderer renderer = new DiscRenderer(context.Parameter))
          {
            return renderer.GetPieAt(point);
          }

        default:
          // TODO: 円グラフ以外の形式はまだ
          return null;
      }
    }

    public PartialGraphImage GetElementImage(IGraphElement[] elements, bool highlighted)
    {
      switch (context.Parameter.Graph.Type)
      {
        case GraphType.Disc:
          using (DiscRenderer renderer = new DiscRenderer(context.Parameter))
          {
            return renderer.GetPieImage(elements, highlighted);
          }

        default:
          // TODO: 円グラフ以外の形式はまだ
          return null;
      }
    }
  }
}
