using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Text;

namespace Smdn.Applications.DirectoryAnalyzer.Render
{
  using Graphing;

  public class RenderingContext : TaskContext, ITaskContext
  {
    public RenderingContext()
    {
      this.parameter = new RenderingParameter();
      this.status = new RenderingStatus();
      this.result = new RenderingResult();
    }

    #region parameters
    private RenderingParameter parameter = null;

    public RenderingParameter Parameter
    {
      get
      {
        return this.parameter;
      }
    }

    public class RenderingParameter : TaskParameter
    {
      public Graph Graph = null;
      public uint OutputWidth = 800;
      public uint OutputHeight = 800;
      public float InnerDiameterPercent = 0.35f;
      public bool DrawNames = true;
      public bool DrawStatistics = true;
      public bool DrawSmallEntries = true;
      public uint MaxDepth = 0;
      public SmoothingMode SmoothingMode = SmoothingMode.AntiAlias;
      public TextRenderingHint TextRenderingHint = TextRenderingHint.AntiAlias;
      public InterpolationMode InterpolationMode = InterpolationMode.HighQualityBicubic;
      public Color BackgroundColor = Color.White;
      public Color TextColor = Color.Black;
    }
    #endregion

    #region status
    private RenderingStatus status = null;

    public RenderingStatus Status
    {
      get
      {
        return this.status;
      }
    }

    public class RenderingStatus : TaskStatus
    {
      public uint TotalEntryCount = 0;
      public uint RenderedEntryCount = 0;

      public override uint Total
      {
        get
        {
          return this.TotalEntryCount;
        }
      }

      public override uint Processed
      {
        get
        {
          return this.RenderedEntryCount;
        }
      }
    }
    #endregion

    #region results
    private RenderingResult result = null;

    public RenderingResult Result
    {
      get
      {
        return this.result;
      }
    }

    public class RenderingResult : TaskResult
    {
      public Bitmap GraphImage = null;
    }
    #endregion

    #region ITaskContext

    TaskContext.TaskParameter ITaskContext.Parameter
    {
      get { return this.parameter; }
    }

    TaskContext.TaskResult ITaskContext.Result
    {
      get { return this.result; }
    }

    TaskContext.TaskStatus ITaskContext.Status
    {
      get { return this.status; }
    }

    #endregion
  }
}
