using System;
using System.Diagnostics;
using System.Reflection;
using System.Text;
using System.Windows.Forms;

namespace Smdn.Applications.DirectoryAnalyzer.UI
{
  public partial class AboutDialog : Form
  {
    public AboutDialog()
    {
      InitializeComponent();

      Initialize();
    }

    private void Initialize()
    {
      this.StartPosition = FormStartPosition.CenterParent;

      AssemblyFileVersionAttribute versionAttr = Attribute.GetCustomAttribute(Assembly.GetExecutingAssembly(), typeof(AssemblyFileVersionAttribute)) as AssemblyFileVersionAttribute;
      AssemblyCopyrightAttribute copyrightAttr = Attribute.GetCustomAttribute(Assembly.GetExecutingAssembly(), typeof(AssemblyCopyrightAttribute)) as AssemblyCopyrightAttribute;
      AssemblyDistributionPageAttribute distributionAttr = Attribute.GetCustomAttribute(Assembly.GetExecutingAssembly(), typeof(AssemblyDistributionPageAttribute)) as AssemblyDistributionPageAttribute;

      labelApplication.Text = string.Format("DirectoryAnalyzer version {0}{1}{2}",
        versionAttr.Version,
        Environment.NewLine,
        copyrightAttr.Copyright);

      linkLabelDistPage.Text = distributionAttr.Url;
      linkLabelDistPage.Links.Add(0, linkLabelDistPage.Text.Length, linkLabelDistPage.Text);
      linkLabelDistPage.LinkClicked += delegate(object sender, LinkLabelLinkClickedEventArgs e)
      {
        e.Link.Visited = true;

        ProcessStartInfo startInfo = new ProcessStartInfo();

        startInfo.FileName = e.Link.LinkData as string;

        try
        {
          using (Process process = Process.Start(startInfo))
          {
            if (null != process) process.Close();
          }
        }
        catch (Exception ex)
        {
          Console.Error.WriteLine("Couldn't open link {0}: {1}", startInfo.FileName, ex);
        }
      };

      StringBuilder sb = new StringBuilder();

      sb.AppendLine("DirectoryAnalyzerではアイコンの一部に'Silk icon'を使用しています。");
      sb.AppendLine();
      sb.AppendLine("Silk icon set 1.3 (famfamfam.com)");
      sb.AppendLine("http://www.famfamfam.com/lab/icons/silk/");

      textBoxNotice.Text = sb.ToString();

      buttonOk.Click += delegate
      {
        this.Close();
      };
    }
  }
}