using System;
using System.Windows.Forms;

namespace Smdn.Applications.DirectoryAnalyzer.UI
{
  using FileSystem;
  using Graphing;

  public partial class MainWindow
  {
    private interface IDirectorySelectionControl
    {
      Directory Directory { get; }
    }

    private class DirectorySelectionToolStripButton : ToolStripButton, IDirectorySelectionControl
    {
      private Directory directory = null;

      public DirectorySelectionToolStripButton(Directory directory)
        : base()
      {
        this.directory = directory;
      }

      public Directory Directory
      {
        get
        {
          return this.directory;
        }
      }

      Directory IDirectorySelectionControl.Directory
      {
        get
        {
          return this.directory;
        }
      }
    }

    private class DirectorySelectionToolStripMenuItem : ToolStripMenuItem, IDirectorySelectionControl
    {
      private Directory directory = null;

      public DirectorySelectionToolStripMenuItem(Directory directory)
        : base()
      {
        this.directory = directory;
      }

      public Directory Directory
      {
        get
        {
          return this.directory;
        }
      }

      Directory IDirectorySelectionControl.Directory
      {
        get
        {
          return this.directory;
        }
      }
    }

    private interface IDeviceSelectionControl
    {
      Device Device { get; }
    }

    private class DeviceSelectionToolStripButton : ToolStripButton, IDeviceSelectionControl
    {
      private Device device = null;

      public DeviceSelectionToolStripButton(Device device)
        : base()
      {
        this.device = device;
      }

      Device IDeviceSelectionControl.Device
      {
        get
        {
          return this.device;
        }
      }
    }

    private class DeviceSelectionToolStripMenuItem : ToolStripMenuItem, IDeviceSelectionControl
    {
      private Device device = null;

      public DeviceSelectionToolStripMenuItem(Device device)
        : base()
      {
        this.device = device;
      }

      Device IDeviceSelectionControl.Device
      {
        get
        {
          return this.device;
        }
      }
    }

    private interface IVisualizerControl
    {
      Visualizer Visualizer { get; }
    }

    private class VisualizedHistoryToolStripButton : ToolStripButton, IVisualizerControl
    {
      private Visualizer visualizer = null;

      Visualizer IVisualizerControl.Visualizer
      {
        get
        {
          return this.visualizer;
        }
      }

      public VisualizedHistoryToolStripButton(Visualizer visualizer)
        : base()
      {
        this.visualizer = visualizer;
      }
    }

    private class VisualizedHistoryToolStripMenuItem : ToolStripMenuItem, IVisualizerControl
    {
      private Visualizer visualizer = null;

      Visualizer IVisualizerControl.Visualizer
      {
        get
        {
          return this.visualizer;
        }
      }

      public VisualizedHistoryToolStripMenuItem(Visualizer visualizer)
        : base()
      {
        this.visualizer = visualizer;
      }

    }

    private class GraphElementContextMenuStrip : ContextMenuStrip
    {
      private IGraphElement element = null;

      public IGraphElement Element
      {
        get
        {
          return this.element;
        }
      }

      public void Show(Control control, int x, int y, IGraphElement element)
      {
        base.Show(control, x, y);

        this.element = element;
      }
    }

    /// <remarks>
    /// this class is based on SantaMarta.Applications.Usamimi.UI.TextBoxes.QueryTextBox
    /// </remarks>
    private class QueryTextBox : TextBox
    {
      public event EventHandler QueryEntered;

      /// <summary>
      /// クエリが入力された
      /// </summary>
      protected virtual void OnQueryEntered(EventArgs e)
      {
        if (null != QueryEntered) QueryEntered(this, e);
      }

      /// <summary>
      /// OnEnter
      /// </summary>
      protected override void OnEnter(EventArgs e)
      {
        base.OnEnter(e);

        if (0 == this.SelectionLength) this.SelectAll();
      }

      /// <summary>
      /// OnClick
      /// </summary>
      protected override void OnClick(EventArgs e)
      {
        if (0 == this.SelectionLength) this.SelectAll();

        base.OnClick(e);
      }

      /// <summary>
      /// ProcessCmdKey
      /// </summary>
      protected override bool ProcessCmdKey(ref Message m, Keys keyData)
      {
#if LINUX
        const int WM_KEYDOWN = 0x100;

        if (WM_KEYDOWN == m.Msg)
        {
          // キー、修飾キー
          Keys key = keyData & Keys.KeyCode;
          //Keys modifiers = keyData & Keys.Modifiers;

          if (key == Keys.Enter)
          {
            OnQueryEntered(EventArgs.Empty);

            return true;
          }
        }

        return base.ProcessCmdKey(ref m, keyData);
#elif WINDOWS
        const int WM_KEYDOWN = 0x100;

        if (WM_KEYDOWN == m.Msg)
        {
          // キー、修飾キー
          Keys key = keyData & Keys.KeyCode;
          //Keys modifiers = keyData & Keys.Modifiers;

          if (key == Keys.Enter)
          {
            OnQueryEntered(EventArgs.Empty);

            return true;
          }
        }

        return base.ProcessCmdKey(ref m, keyData);
#endif
      }
    }
  }
}
