using System;
using System.Windows.Forms;

namespace Smdn.Applications.DirectoryAnalyzer.UI
{
  using FileSystem;
  using Graphing;

  public partial class Tabulator
  {
    private class ElementListViewItem : ListViewItem
    {
      private IGraphElement element = null;

      public IGraphElement Element
      {
        get
        {
          return this.element;
        }
      }

      protected ElementListViewItem(IGraphElement element)
      {
        this.element = element;
      }
    }

    private class DirectoryListViewItem : ElementListViewItem
    {

      public DirectoryListViewItem(IGraphElement element)
        : base(element)
      {
        Directory directory = element.Entry as Directory;

        this.Text = directory.Name + System.IO.Path.DirectorySeparatorChar;
        this.SubItems.Add("d");
        this.SubItems.Add(element.Range.ToString("P2"));
        this.SubItems.Add(directory.Size.ToBinaryPrefixedString());
        this.SubItems.Add(directory.FullName);
      }
    }

    private class FileListViewItem : ElementListViewItem
    {
      public FileListViewItem(IGraphElement element)
        : base(element)
      {
        File file = element.Entry as File;

        this.Text = file.Name;
        this.SubItems.Add("f");
        this.SubItems.Add(element.Range.ToString("P2"));
        this.SubItems.Add(file.Size.ToBinaryPrefixedString());
        this.SubItems.Add(file.FullName);
      }
    }

    private class EntryListViewColumn : ColumnHeader
    {
      private EntryCompareMode compareMode = EntryCompareMode.BySize;

      public EntryCompareMode CompareMode
      {
        get
        {
          return this.compareMode;
        }
      }

      public EntryListViewColumn(string text, int width, EntryCompareMode compareMode)
        : base()
      {
        this.compareMode = compareMode;

        base.Text = text;
        base.Width = width;
      }
    }
  }
}
