using System;

#if LINUX
using System.Diagnostics;
using System.Text;
using System.Windows.Forms;
#elif WINDOWS
using System.Diagnostics;
using System.Runtime.InteropServices;
#endif

namespace Smdn.Applications.DirectoryAnalyzer.Util
{
  using FileSystem;

  public static class FileManager
  {
    public static void OpenEntry(Entry entry)
    {
#if LINUX
      OpenByFileManager(entry.FullName);
#elif WINDOWS
      if (entry is File)
      {
        ShellExecute(entry as File);
      }
      else if (entry is Directory)
      {
        ExploreEntry(entry, false);
      }
#endif
    }

    public static void OpenParentEntry(Entry entry)
    {
#if LINUX
      if (null == entry.Parent)
      {
        OpenByFileManager(entry.FullName);
      }
      else
      {
        OpenByFileManager(entry.Parent.FullName);
      }
#elif WINDOWS
      if (entry is Directory && null == (entry as Directory).Parent)
      {
        ExploreEntry(entry, false);
      }
      else
      {
        ExploreEntry(entry, true);
      }
#endif
    }

    public static void ShowProperty(Entry entry)
    {
#if LINUX
      // TODO: プロパティダイアログを開きたい
      StringBuilder sb = new StringBuilder();

      sb.AppendLine(string.Format("FileType: {0}", entry.Info.FileType));
      sb.AppendLine(string.Format("Length: {0}", entry.Info.Length));
      sb.AppendLine(string.Format("OwnerUser: {0}", entry.Info.OwnerUser));
      sb.AppendLine(string.Format("OwnerGroup: {0}", entry.Info.OwnerGroup));
      sb.AppendLine(string.Format("FileAccessPermissions: {0}", entry.Info.FileAccessPermissions));
      sb.AppendLine(string.Format("FileSpecialAttributes: {0}", entry.Info.FileSpecialAttributes));
      sb.AppendLine(string.Format("Protection: {0}", entry.Info.Protection));
      sb.AppendLine(string.Format("BlockSize: {0}", entry.Info.BlockSize));
      sb.AppendLine(string.Format("DeviceType: {0}", entry.Info.DeviceType));
      sb.AppendLine(string.Format("LastAccessTime: {0}", entry.Info.LastAccessTime));
      sb.AppendLine(string.Format("LastWriteTime: {0}", entry.Info.LastWriteTime));
      sb.AppendLine(string.Format("LastStatusChangeTime: {0}", entry.Info.LastStatusChangeTime));

      MessageBox.Show(sb.ToString(), entry.Name, MessageBoxButtons.OK);
#elif WINDOWS
      ShowPropertyDialog(entry);
#endif
    }

#if LINUX
    private static void OpenByFileManager(string path)
    {
      Console.WriteLine(path);

      ProcessStartInfo startInfo = new ProcessStartInfo();

      startInfo.FileName = string.Format("file://{0}", path);

      try
      {
        using (Process process = Process.Start(startInfo))
        {
          if (null != process) process.Close();
        }
      }
      catch (Exception ex)
      {
        Console.Error.WriteLine("Couldn't open path {0}: {1}", path, ex);
      }
    }
#elif WINDOWS
    [DllImport("shell32.dll")]
    private static extern bool SHObjectProperties(IntPtr hwnd,
                                                  uint shopObjectType,
                                                  [MarshalAs(UnmanagedType.LPWStr)] string pszObjectName,
                                                  [MarshalAs(UnmanagedType.LPWStr)] string pszPropertyPage);

    private const uint SHOP_PRINTERNAME = 0x1;
    private const uint SHOP_FILEPATH = 0x2;
    private const uint SHOP_VOLUMEGUID = 0x4;

    private static void ShellExecute(File file)
    {
      ProcessStartInfo startInfo = new ProcessStartInfo();

      startInfo.FileName = file.FullName;
      startInfo.ErrorDialog = true;
      startInfo.UseShellExecute = true;

      try
      {
        using (Process process = Process.Start(startInfo))
        {
          if (null != process) process.Close();
        }
      }
      catch (System.ComponentModel.Win32Exception)
      {
      }
    }

    private static void ExploreEntry(Entry entry, bool selected)
    {
      ProcessStartInfo startInfo = new ProcessStartInfo();

      startInfo.FileName = "explorer.exe";
      startInfo.Arguments = entry.FullName;
      startInfo.UseShellExecute = false;

      if (selected)
      {
        startInfo.Arguments = "/select," + startInfo.Arguments;
      }

      using (Process process = Process.Start(startInfo))
      {
        if (null != process) process.Close();
      }
    }

    /// <remarks>
    /// this method is based on SantaMarta.Win32APIWrapper.Shell.FileProperties
    /// </remarks>
    private static void ShowPropertyDialog(Entry entry)
    {
      SHObjectProperties(IntPtr.Zero,
                         SHOP_FILEPATH,
                         entry.FullName,
                         string.Empty);
    }
#endif
  }
}
