#if LINUX

using Gtk;
using System;
using System.IO;
using System.Drawing;

namespace Smdn.Applications.DirectoryAnalyzer.Util
{
  using FileSystem;

  public static partial class Icon
  {
    private static Bitmap LookupIcon(string iconName)
    {
      IconInfo info = IconTheme.Default.LookupIcon(iconName, 16, IconLookupFlags.NoSvg);
      Bitmap icon = null;

      if (null == info)
      {
        Console.Error.WriteLine("Icon '{0}' not found", iconName);
      }
      else
      {
        FileStream stream = null;

        try
        {
          stream = new FileStream(info.Filename, FileMode.Open, FileAccess.Read);

          icon = new Bitmap(stream);
        }
        catch(Exception ex)
        {
          Console.Error.WriteLine("Couldn't load icon {0}: {1}", info.DisplayName, ex);

          icon = null;
        }
        finally
        {
          if (null != stream) stream.Close();
        }
      }

      if (null == icon)
      {
        return DefaultIcon;
      }
      else
      {
        return icon;
      }
    }

    private static Bitmap GetIcon_Linux(IconType type)
    {
      switch (type)
      {
        case IconType.Folder:
          return LookupIcon("folder");

        case IconType.DesktopFolder:
          return LookupIcon("desktop");

        case IconType.MyDocumentFolder:
          return LookupIcon("folder_home");

        case IconType.FolderSelection:
          return LookupIcon("fileopen");
      }

      return DefaultIcon;
    }

    private static Bitmap GetIcon_Linux(Directory directory)
    {
      if (System.IO.Path.Equals(Environment.GetFolderPath(Environment.SpecialFolder.Desktop), directory.FullName))
      {
        return GetIcon_Linux(IconType.DesktopFolder);
      }
      else if (System.IO.Path.Equals(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments), directory.FullName))
      {
        return GetIcon_Linux(IconType.MyDocumentFolder);
      }
      else
      {
        return GetIcon_Linux(IconType.Folder);
      }
    }

    private static Bitmap GetIcon_Linux(DeviceType type)
    {
      return GetIcon_Linux(IconType.Folder);
    }
  }
}
#endif
