#if WINDOWS

using System;
using System.Drawing;

namespace Smdn.Applications.DirectoryAnalyzer.Util
{
  using FileSystem;
  using Util.Windows;

  public static partial class Icon
  {
    private static readonly IntPtr hInstance = System.Runtime.InteropServices.Marshal.GetHINSTANCE(System.Reflection.Assembly.GetExecutingAssembly().GetModules()[0]);
    private static readonly string pathToShell32dll = Environment.ExpandEnvironmentVariables("%systemroot%\\system32\\shell32.dll");
    private static readonly string pathToMydocsdll = Environment.ExpandEnvironmentVariables("%systemroot%\\system32\\mydocs.dll");
    private static readonly string pathToExplorer = Environment.ExpandEnvironmentVariables("%systemroot%\\explorer.exe");

    private static Bitmap GetIconFromFile(string file, int index)
    {
      Windows.Icon icon = AssociatedIcon.Extract(file, index, AssociatedIcon.IconSize.Small, hInstance);

      if (null == icon)
      {
        return DefaultIcon;
      }
      else
      {
        return IconUtil.CreateBitmap(icon, true);
      }
    }

    private static Bitmap GetIconFromShell32(int index)
    {
      return GetIconFromFile(pathToShell32dll, index);
    }

    private static Bitmap GetIcon_Windows(IconType type)
    {
      switch (type)
      {
        case IconType.Folder:
          return GetIconFromFile(pathToExplorer, 13);

        case IconType.DesktopFolder:
          return GetIconFromFile(pathToExplorer, 3);

        case IconType.MyDocumentFolder:
          return GetIconFromFile(pathToMydocsdll, 0);

        case IconType.ProgramFilesFolder:
          return GetIconFromShell32(19);

        case IconType.FolderSelection:
          return GetIconFromShell32(22);
      }

      return DefaultIcon;
    }

    private static Bitmap GetIcon_Windows(Directory directory)
    {
      if (System.IO.Path.Equals(Environment.GetFolderPath(Environment.SpecialFolder.Desktop), directory.FullName))
      {
        return GetIcon_Windows(IconType.DesktopFolder);
      }
      else if (System.IO.Path.Equals(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments), directory.FullName))
      {
        return GetIcon_Windows(IconType.MyDocumentFolder);
      }
      else if (System.IO.Path.Equals(Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles), directory.FullName))
      {
        return GetIcon_Windows(IconType.ProgramFilesFolder);
      }
      else
      {
        return GetIcon_Windows(IconType.Folder);
      }
    }

    private static Bitmap GetIcon_Windows(DeviceType type)
    {
      switch (type)
      {
        case DeviceType.Fixed:
          return GetIconFromShell32(8);

        case DeviceType.CDRom:
          return GetIconFromShell32(11);

        case DeviceType.Network:
          return GetIconFromShell32(9);

        case DeviceType.Removable:
          return GetIconFromShell32(7);

        default:
          // フォルダのアイコンで代用
          return GetIcon_Windows(IconType.Folder);
      }
    }
  }
}
#endif
