#if WINDOWS

using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;

namespace Smdn.Applications.DirectoryAnalyzer.Util.Windows
{
  public static class IconUtil
  {
    public static Bitmap CreateBitmap(Windows.Icon icon, bool includeAlphaChannel)
    {
      Bitmap b;
      BitmapData bitmapData = null;

      if (includeAlphaChannel)
      {
        b = new Bitmap(icon.Width, icon.Height, PixelFormat.Format32bppArgb);
      }
      else
      {
        return CreateImageBitmap(icon);
      }

      try
      {
        bitmapData = b.LockBits(new Rectangle(0, 0, icon.Width, icon.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);

        IntPtr scan0 = bitmapData.Scan0;
        int stride = bitmapData.Stride;
        Windows.Icon.Argb[,] image = icon.Image;
        byte[,] mask = icon.Mask;

        // 色部分の出力
        for (int y = 0; y < icon.Height; y++)
        {
          for (int x = 0; x < icon.Width; x++)
          {
            int imageIndex = stride * y + x * 4;

            Marshal.WriteByte(scan0, imageIndex, image[x, y].B);
            Marshal.WriteByte(scan0, imageIndex + 1, image[x, y].G);
            Marshal.WriteByte(scan0, imageIndex + 2, image[x, y].R);
            Marshal.WriteByte(scan0, imageIndex + 3, (byte)(image[x, y].A & mask[x, y]));
          }
        }
      }
      finally
      {
        if (bitmapData != null)
        {
          b.UnlockBits(bitmapData);
        }
      }

      return b;
    }

    public static Bitmap CreateImageBitmap(Windows.Icon icon)
    {
      Bitmap b = new Bitmap(icon.Width, icon.Height, PixelFormat.Format24bppRgb);
      BitmapData bitmapData = null;

      try
      {
        bitmapData = b.LockBits(new Rectangle(0, 0, icon.Width, icon.Height), ImageLockMode.WriteOnly, PixelFormat.Format24bppRgb);

        IntPtr scan0 = bitmapData.Scan0;
        int stride = bitmapData.Stride;
        Windows.Icon.Argb[,] image = icon.Image;

        // 色部分の出力
        for (int y = 0; y < icon.Height; y++)
        {
          for (int x = 0; x < icon.Width; x++)
          {
            int imageIndex = stride * y + x * 3;

            Marshal.WriteByte(scan0, imageIndex, image[x, y].B);
            Marshal.WriteByte(scan0, imageIndex + 1, image[x, y].G);
            Marshal.WriteByte(scan0, imageIndex + 2, image[x, y].R);
          }
        }
      }
      finally
      {
        if (bitmapData != null)
        {
          b.UnlockBits(bitmapData);
        }
      }

      return b;
    }

    public static Bitmap CreateAlphaBitmap(Windows.Icon icon)
    {
      Bitmap b = new Bitmap(icon.Width, icon.Height, PixelFormat.Format24bppRgb);
      BitmapData bitmapData = null;

      try
      {
        bitmapData = b.LockBits(new Rectangle(0, 0, icon.Width, icon.Height), ImageLockMode.WriteOnly, PixelFormat.Format24bppRgb);

        IntPtr scan0 = bitmapData.Scan0;
        int stride = bitmapData.Stride;
        Windows.Icon.Argb[,] image = icon.Image;

        // 色部分の出力
        for (int y = 0; y < icon.Height; y++)
        {
          for (int x = 0; x < icon.Width; x++)
          {
            int imageIndex = stride * y + x * 4;

            Marshal.WriteByte(scan0, imageIndex, image[x, y].A);
            Marshal.WriteByte(scan0, imageIndex + 1, image[x, y].A);
            Marshal.WriteByte(scan0, imageIndex + 2, image[x, y].A);
          }
        }
      }
      finally
      {
        if (bitmapData != null)
        {
          b.UnlockBits(bitmapData);
        }
      }

      return b;
    }

    public static Bitmap CreateMaskBitmap(Windows.Icon icon)
    {
      Bitmap b = new Bitmap(icon.Width, icon.Height, PixelFormat.Format24bppRgb);
      BitmapData bitmapData = null;

      try
      {
        bitmapData = b.LockBits(new Rectangle(0, 0, icon.Width, icon.Height), ImageLockMode.WriteOnly, PixelFormat.Format24bppRgb);

        IntPtr scan0 = bitmapData.Scan0;
        int stride = bitmapData.Stride;
        byte[,] mask = icon.Mask;

        // 色部分の出力
        for (int y = 0; y < icon.Height; y++)
        {
          for (int x = 0; x < icon.Width; x++)
          {
            int imageIndex = stride * y + x * 3;

            if (mask[x, y] == 0)
            {
              Marshal.WriteByte(scan0, imageIndex, 0x00);
              Marshal.WriteByte(scan0, imageIndex + 1, 0x00);
              Marshal.WriteByte(scan0, imageIndex + 2, 0x00);
            }
            else
            {
              Marshal.WriteByte(scan0, imageIndex, 0xff);
              Marshal.WriteByte(scan0, imageIndex + 1, 0xff);
              Marshal.WriteByte(scan0, imageIndex + 2, 0xff);
            }
          }
        }
      }
      finally
      {
        if (bitmapData != null)
        {
          b.UnlockBits(bitmapData);
        }
      }

      return b;
    }
  }
}
#endif
