#if WINDOWS

using System;
using System.Runtime.InteropServices;

namespace Smdn.Applications.DirectoryAnalyzer.Util.Windows
{
  [StructLayout(LayoutKind.Sequential, Pack = 4)]
  public struct ICONDIR
  {
    public ushort idReserved;
    public ushort idType;
    public ushort idCount;
    public ICONDIRENTRY[] idEntries;
    public ICONIMAGE[] idImages;
  }

  [StructLayout(LayoutKind.Sequential, Pack = 4)]
  public struct ICONDIRENTRY
  {
    public byte bWidth;
    public byte bHeight;
    public byte bColorCount;
    public byte bReserved;
    public ushort wPlanes;
    public ushort wBitCount;
    public uint dwBytesInRes;
    public uint dwImageOffset;

  }

  [StructLayout(LayoutKind.Sequential, Pack = 4)]
  public struct BITMAPFILEHEADER
  {
    public ushort bfType;
    public uint bfSize;
    public ushort bfReserved1;
    public ushort bfReserved2;
    public uint bfOffBits;
  }

  [StructLayout(LayoutKind.Sequential, Pack = 4)]
  public struct BITMAPINFOHEADER
  {
    public uint biSize;
    public int biWidth;
    public int biHeight;
    public ushort biPlanes;
    public ushort biBitCount;
    public uint biCompression;
    public uint biSizeImage;
    public int biXPelsPerMeter;
    public int biYPelsPerMeter;
    public uint biClrUsed;
    public uint biClrImportant;
  }

  [StructLayout(LayoutKind.Explicit)]
  public struct RGBQUAD
  {
    [FieldOffset(0)]
    public uint rgbValue;

    [FieldOffset(0)]
    public byte rgbBlue;
    [FieldOffset(1)]
    public byte rgbGreen;
    [FieldOffset(2)]
    public byte rgbRed;
    [FieldOffset(3)]
    public byte rgbReserved;
  }

  [StructLayout(LayoutKind.Sequential, Pack = 4)]
  public struct ICONIMAGE
  {
    public BITMAPINFOHEADER icHeader;
    public RGBQUAD[] icColors;
    public byte[] icXor;
    public byte[] icAnd;
  }

  [StructLayout(LayoutKind.Sequential, Pack = 4)]
  public struct ICONINFO
  {
    [MarshalAs(UnmanagedType.Bool)]
    public bool fIcon;

    public uint xHotspot;
    public uint yHotspot;
    public IntPtr hbmMask;
    public IntPtr hbmColor;
  }

  [StructLayout(LayoutKind.Sequential, Pack = 4)]
  public struct BITMAP
  {
    public int bmType;
    public int bmWidth;
    public int bmHeight;
    public int bmWidthBytes;
    public ushort bmPlanes;
    public ushort bmBitsPixel;
    public IntPtr bmBits;
  }

  [StructLayout(LayoutKind.Sequential, Pack = 4)]
  public struct DIBSECTION
  {
    public BITMAP dsBm;
    public BITMAPINFOHEADER dsBmih;
    public uint dsBitfields0;
    public uint dsBitfields1;
    public uint dsBitfields2;
    public uint dsBitfields3;
    public IntPtr dshSection;
    public uint dsOffset;
  }

  [StructLayout(LayoutKind.Sequential, Pack = 4)]
  public struct BITMAPINFO_1BPP
  {
    public BITMAPINFOHEADER bmiHeader;
    public RGBQUAD bmiColor1;
    public RGBQUAD bmiColor2;
  }

  [StructLayout(LayoutKind.Sequential, Pack = 4)]
  public struct BITMAPINFO_32BPP
  {
    public BITMAPINFOHEADER bmiHeader;
  }

  public partial class PlatformApi
  {
    public const uint DIB_RGB_COLORS = 0;
    public const uint DIB_PAL_COLORS = 1;
    public const uint DIB_PAL_INDICES = 2;

    [DllImport("user32.dll", SetLastError = true)]
    [return: MarshalAs(UnmanagedType.Bool)]
    public static extern bool GetIconInfo(IntPtr hIcon, out ICONINFO piconinfo);

    [DllImport("gdi32.dll", SetLastError = true)]
    public static extern int GetObject(IntPtr handle, int cbBuffer, ref BITMAP lpvObject);

    [DllImport("gdi32.dll", SetLastError = true)]
    public static extern int GetObject(IntPtr handle, int cbBuffer, ref DIBSECTION lpvObject);

    [DllImport("gdi32.dll", SetLastError = true)]
    public static extern int GetDIBits(IntPtr hdc, IntPtr hbmp, uint uStartScan, uint cScanLines, IntPtr lpvBits, ref BITMAPINFO_32BPP lpbmi, uint uUsage);

    [DllImport("gdi32.dll", SetLastError = true)]
    public static extern int GetDIBits(IntPtr hdc, IntPtr hbmp, uint uStartScan, uint cScanLines, IntPtr lpvBits, ref BITMAPINFO_1BPP lpbmi, uint uUsage);

    [DllImport("gdi32.dll", SetLastError = true)]
    public static extern int GetDIBits(IntPtr hdc, IntPtr hbmp, uint uStartScan, uint cScanLines, IntPtr lpvBits, ref IntPtr lpbmi, uint uUsage);
  }
}
#endif
