#if WINDOWS

using System;
using System.Text;
using System.Runtime.InteropServices;

namespace Smdn.Applications.DirectoryAnalyzer.Util.Windows
{
  public class PlatformApiException : Exception
  {
    public PlatformApiException()
      : this(null, Marshal.GetLastWin32Error())
    {
    }

    public PlatformApiException(int errorCode)
      : this(null, errorCode)
    {
    }

    public PlatformApiException(string message)
      : this(message, Marshal.GetLastWin32Error())
    {
    }

    public PlatformApiException(string message, int errorCode)
      : base(GetErrorMessage(message, errorCode))
    {
    }

    private static string GetErrorMessage(string message, int errorCode)
    {
      string error = string.Empty;
      IntPtr lpBuffer = IntPtr.Zero;

      if (0 == PlatformApi.FormatMessage(
                      PlatformApi.FORMAT_MESSAGE_ALLOCATE_BUFFER | PlatformApi.FORMAT_MESSAGE_FROM_SYSTEM | PlatformApi.FORMAT_MESSAGE_IGNORE_INSERTS,
                      IntPtr.Zero,
                      errorCode,
                      0,
                      ref lpBuffer,
                      0,
                      IntPtr.Zero))
      {
        error = string.Format("エラーコード: {0}", errorCode);
      }
      else
      {
        error = Marshal.PtrToStringAuto(lpBuffer);

        Marshal.FreeHGlobal(lpBuffer);

        error = error.TrimEnd('\n', '\r');
      }

      if (string.IsNullOrEmpty(message))
      {
        return string.Format("{0}　'{1}'", message, error);
      }
      else
      {
        return string.Format("プラットフォームAPIの呼び出し中にエラーが発生しました。　'{0}'", message, error);
      }
    }
  }
}
#endif